/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "stepup.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define STEPUP_DUMMY 0

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void stepup_spi_set ( stepup_t *ctx, uint16_t command );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void stepup_cfg_setup ( stepup_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->mod = HAL_PIN_NC;
    cfg->en = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;

    cfg->config_word_cfg = STEPUP_DACA | STEPUP_BUFFERED | STEPUP_GAIN_1X | STEPUP_POWER_UP;
}

STEPUP_RETVAL stepup_init ( stepup_t *ctx, stepup_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    ctx->config_word = cfg->config_word_cfg;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return STEPUP_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, STEPUP_DUMMY );

    // Output pins 
    
    digital_out_init( &ctx->mod, cfg->mod );
    digital_out_init( &ctx->en, cfg->en );

    spi_master_stop( &ctx->spi );
    digital_out_high( &ctx->mod );
    digital_out_high( &ctx->en );

    return STEPUP_OK;

}

void stepup_default_cfg ( stepup_t *ctx, uint16_t out_value )
{
    // Click default configuration 
    digital_out_low( &ctx->mod );
    digital_out_high( &ctx->en );

    stepup_set_out( ctx, out_value );
}

void stepup_generic_transfer ( stepup_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start( &ctx->spi );
    spi_master_transfer( &ctx->spi, block );
    spi_master_stop( &ctx->spi );    
}

void stepup_mod_set ( stepup_t *ctx, uint8_t pin_state )
{
    digital_out_write( &ctx->mod, pin_state );
}

void stepup_en_set ( stepup_t *ctx, uint8_t pin_state )
{
    digital_out_write( &ctx->en, pin_state );
}

uint8_t stepup_set_config ( stepup_t *ctx, uint16_t config )
{
    if ( ( config & 0x0FFF ) != 0 ) 
    {
        return 1;
    }

    ctx->config_word = config;
    
    return 0;
}

uint8_t stepup_set_out ( stepup_t *ctx, uint16_t out_value )
{
    uint16_t aux_value;
    
    if ( out_value > 4095 )
    {
        return 1;
    }

    aux_value = ctx->config_word;
    aux_value |= out_value;
    stepup_spi_set( ctx, aux_value );
    
    return 0;
}

float stepup_get_percent ( uint16_t out_value )
{
    float aux_out;
    
    aux_out  = ( float )( out_value ) / 4095;
    aux_out *= 100;
    aux_out  = ( float )( 100 - aux_out );
    
    return aux_out;
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void stepup_spi_set ( stepup_t *ctx, uint16_t command )
{
    uint8_t aux_buffer[ 2 ];

    aux_buffer[ 0 ] = command >> 8 ;
    aux_buffer[ 1 ] = command & 0x00FF;

    spi_master_start( &ctx->spi );
    spi_master_write( &ctx->spi, aux_buffer[ 0 ] );
    spi_master_write( &ctx->spi, aux_buffer[ 1 ] );
    spi_master_stop( &ctx->spi );  
}

// ------------------------------------------------------------------------- END

